@@ -17,18 +17,11 @@ from abc import ABC, abstractproperty
 
 
 
-
-# =============================================================================
-# Problems:
-# =============================================================================
-# Summarize Nodes?
-# Summarize Loads?
-
-
 class Section2D():
     E:None
     A:None
     Ixx:None
+    Iyy:None
     
 
 
@@ -36,17 +29,19 @@ class Section2D():
 class SectionBasic2D(Section2D):
     """
     A basic section that contains the global propreties of the beam section,
-    without any geometry.
+    without any geometry. It's assume the section is elastic.
     """
     E:float = 1
     A:float = 1
     Ixx:float = 1
+    Iyy:float = 1
 
 
 @dataclass
 class SectionRectangle(Section2D):
     """
-    Represents a Rectangular section.
+    Represents a elastic Rectangular section. Ixx and A are calcualted using 
+    the beam width and height.
     """
     
     E:float = 200*10**9
@@ -57,32 +52,37 @@ class SectionRectangle(Section2D):
     def __post_init__(self):
         self.A = self.d*self.w
         self.Ixx = self.d**3*self.w / 12
+        self.Ixx = self.w**3*self.d / 12
 
         
 class Node2D(NodeArchetype):
-    def __init__(self, x, fixity, label = None):
-        """
-        Represents a node.
-        Nodes have labels and IDs. 
-        The Labe is a name the user assigns to the node and will be displayed
-        in plots.
-        
-        The ID is a unique name that OpenSees will read. ID - 1 will be the 
-        position in the beam node array
+    """
+    Represents a node.
+    Nodes have labels and IDs. 
+    The Labe is a name the user assigns to the node and will be displayed
+    in plots.
+    
+    The ID is a unique name that OpenSees will read. ID - 1 will be the 
+    position in the beam node array. As new nodes are added, the IDs will be 
+    sorted and updated so that they are always increasing from left to right.
 
-        Parameters
-        ----------
-        x : float
-            The postion of the node.
-        fixity : list
-            A list of the input fixities for nodes.
-        pointLoad : PointLoad
-            DESCRIPTION.
-        label : TYPE, optional
-            DESCRIPTION. The default is None.
-
-        """
-        
+    Parameters
+    ----------
+    x : float
+        The postion of the node.
+    fixity : list
+        A list of the input fixities for each possible degree of freedom. 
+        Each node will have three degree of freedoms; [x, y, :math:`\\theta`]
+        1 represents a fixed condition, 0 represents a free conditon. 
+        e.x. [1,1,0]
+        A pin conneciton that's fixed in x/y but free in rotation.
+    label : str, optional
+        A name for the node. This can be displayed in the plots. The default is ''.
+    """    
+    
+    _dimension = '2D'
+    
+    def __init__(self, x, fixity, label = ''):
         
         self.x = x
         self.fixity = fixity
@@ -98,7 +98,18 @@ class Node2D(NodeArchetype):
         if np.any(fixity != [0,0,0]):
             self.hasReaction = True
 
-    def setID(self, newID):
+    def _setID(self, newID):
+        """
+        Sets the node ID. The node ID is a unique identifier that can be used.
+        Users can use this to set the ID, but the old ID will be replaced 
+        every time the sort function is called.
+
+        Parameters
+        ----------
+        newID : int
+            A unique integer that represents the node.
+            
+        """
         self.ID  = newID
         
     def __repr__(self):
@@ -106,9 +117,9 @@ class Node2D(NodeArchetype):
 
     def getFixityType(self):
         """
-        ???
-        This isn't great, but we can't hash lists so I'm not sure how to 
-        improve it.
+        Returns the type of beam fixity for supported 2D fixities.
+        Currently only free, roller, pinned, and fixed are supported.
+
         """
         if list(self.fixity) == [0,0,0]:
             return 'free'
@@ -142,7 +153,7 @@ class Node2D(NodeArchetype):
 class Beam2D():
     """
     A representation of a beam object, that can be used to define information
-    about basic beams.
+    about basic beams. Units must form a consist unit basis for FEM analysis.
     """
 
     def _initArrays(self):
@@ -153,11 +164,30 @@ class Beam2D():
         self.nodeCoords = set()
         
     def getLength(self):
+        """
+        Returns the length of the beam.
+
+        Returns
+        -------
+        flaot
+            The beam length.
+
+        """
         return self.nodes[-1].x - self.nodes[0].x
     def getxLims(self):
+        """
+        Returns the  of the beam.
+
+        Returns
+        -------
+        list[float]
+            A list with the left most and right most point.
+
+        """        
+        
         return self.nodes[0].x, self.nodes[-1].x
     
-    def sortNodes(self):
+    def _sortNodes(self):
         """
         Sorts and renames the nodes based on their x coordinate.
         This is expensive. Try to minimize calls to it.
@@ -167,42 +197,48 @@ class Beam2D():
         for ii, node in enumerate(self.nodes):
             xcoords[ii] = node.x
             labels[ii]  = node.label
-        # print(labels)
         
         # subtract 1 because the indexes are one less.
         oldInd = np.array(self.getNodeIDs())
-        # print(oldInd)
         
         # Sort the nodes.
-        # print(xcoords)
         sortedInd   = np.argsort(xcoords)
         sortedNodes = np.array(self.nodes)[sortedInd]
         self.nodes  = list(sortedNodes)
         
-        # print(sortedInd)
-        self.resetNodeID()
-        self.remapLabels(sortedInd)
-        self.remapPointLoads(oldInd, sortedInd)    
+        self._resetNodeID()
+        self._remapLabels(sortedInd)
+        self._remapPointLoads(oldInd, sortedInd)    
         
     def getNodeIDs(self):
+        """
+        Gets all of the node IDs.
+
+        Returns
+        -------
+        IDs : list[int]
+            a list of all of the node IDs in the model.
+
+        """
         
         IDs = [None]*self.Nnodes
         for ii, node in enumerate(self.nodes):
             IDs[ii] = node.ID
-            # print(node.ID)
         
         return IDs
            
-    def resetNodeID(self):
+    def _resetNodeID(self):
         """
         Resets all the node IDs based on their position. The ID starts counting
-        at the left-most label
+        at the left-most label.
         """
         for ii, node in enumerate(self.nodes):
-            node.setID(int(ii + 1))
+            node._setID(int(ii + 1))
 
-    def remapPointLoads(self, oldIDs, sortedInd):
+    def _remapPointLoads(self, oldIDs, sortedInd):
         """
+        Remaps the point node pointer to the new node positionl. When the nodes
+        are sorted the old pointer will not reference the correct positon.
         We don't know where the position is because of the newly added node,
         so we have to search each node and find the position.
         """        
@@ -211,9 +247,13 @@ class Beam2D():
             newInd = np.where(sortedIDs == pointLoad.nodeID)[0][0] + 1
             pointLoad.nodeID = newInd
         
-    def remapLabels(self, sortedInd):
+    def _remapLabels(self, sortedInd):
+        """
+        Remaps the label objects so that they point towards the correct node.
+        """
         oldLabels = self.nodeLabels
         newLabel = {}
+        
         for label in oldLabels:
             newLabel[label] = sortedInd[oldLabels[label]]
         self.nodeLabels = newLabel   
@@ -221,20 +261,23 @@ class Beam2D():
     def _addedNodeMessage(self, x):
         print(f'New node added at: {x}')
     
-    def addNode(self, x, fixity = np.array([0.,0.,0.], int), label = None, sort = True):
+    def addNode(self, x, fixity = np.array([0.,0.,0.], int), label = '', sort = True):
         """
-        Adds a new node to the model builder.
+        Adds a new node to the beam. Keyword arguments are passed to the node.
+        See :py:class:`Node2D` for more details
 
         Parameters
         ----------
         x : float
             The x coordinate of the node.
-        fixity : np.array
-            The fixity array. Contains 3 values, one for each dof in order
-            x, y, rotation. 1 means the system is fixed in the DOF of question, 
-            0 means the node is free in the DOF.
-        label : int, optional
-            The unique label of the node in question. 
+        fixity : list
+            A list of the input fixities for each possible degree of freedom. 
+            Each node will have three degree of freedoms; [x, y, :math:`\\theta`]
+            1 represents a fixed condition, 0 represents a free conditon. 
+            e.x. [1,1,0]
+            A pin conneciton that's fixed in x/y but free in rotation.
+        label : str, optional
+            A name for the node. This can be displayed in the plots. The default is ''.
 
         Returns
         -------
@@ -252,7 +295,7 @@ class Beam2D():
             """
             index = self._findNode(x)
             nodeID = self.nodes[index].ID
-            newNode.setID(nodeID)
+            newNode._setID(nodeID)
             
             if label is not None:
                 newNode.label = label
@@ -266,21 +309,23 @@ class Beam2D():
         return -1
 
 
-    def addLabel(self, x, label = None, sort = True):
+    def addLabel(self, x, label, sort = True):
         """
-        Adds a new node to the model builder.
+        Adds a label to the beam at the coordinate in question. If a node exists
+        at this location the label is added to it. 
+        If no node exists at location x, a new node is added.
+        The new node will have default fixity.
 
         Parameters
         ----------
         x : float
             The x coordinate of the node.
-        fixity : np.array
-            The fixity array. Contains 3 values, one for each dof in order
-            x, y, rotation. 1 means the system is fixed in the DOF of question, 
-            0 means the node is free in the DOF.
-        label : int, optional
-            The unique label of the node in question. 
-
+        label : str, optional
+            A name for the node. This can be displayed in the plots. The default is ''.            
+        sort : bool, optional
+            A switch which turns on or off sorting of the nodes after a lable is
+            added. The default value is True, which sorts the nodes.
+        
         Returns
         -------
         flag: int
@@ -304,11 +349,11 @@ class Beam2D():
 
     def _addNewNode(self, newNode:Node2D, sort=True):
         self.Nnodes += 1
-        newNode.setID(self.Nnodes)
+        newNode._setID(self.Nnodes)
         self.nodes.append(newNode)
         self.nodeCoords.add(newNode.x)
         if sort:
-            self.sortNodes()
+            self._sortNodes()
         
 
     def addNodes(self, xCoords, fixities = None, labels = None ):
@@ -330,6 +375,9 @@ class Beam2D():
             New node 1: fixed in x, fixed in y, free in rotation.
             New node 2: free in all DOF.
 
+        label : list[str], optional
+            A list of the labels for each node. 
+            labels are displayed in the plots. The default is ''.
 
         """
                
@@ -344,13 +392,13 @@ class Beam2D():
         for ii in range(newNoads):
             self.addNode(xCoords[ii], fixities[ii], labels[ii], sort)     
 
-        self.sortNodes()
+        self._sortNodes()
         
     def _checkfixityInput(self, fixity):
         
         """
-        Confirm that the appropriate input has been supplied to the fixity
-        vector
+        Confirms that the appropriate input has been supplied to the fixity
+        vector.
         """
                 
         if set(fixity).issubset({0,1}) != True:
@@ -370,17 +418,22 @@ class Beam2D():
 
     def setFixity(self, x, fixity, label = None):
         """
-        Sets the node fixity. If the node exists, update it. If the node doesn't
+        Sets the the model fixity at locaiton x. If the node exists, update it. If the node doesn't
         exist, then a new node will be added
 
         Parameters
         ----------
-        x : TYPE
+        x : float
             The x coordinant of the noded to be modified/added.
-        fixity : int/list/array
-            Either the integer 0/1, or a list of zeros/ones. If equal to 0, 
-            that DOF is considered Free. Otherwise it is considered fixed.
-
+        fixity : list
+            A list of the input fixities for each possible degree of freedom. 
+            Each node will have three degree of freedoms; [x, y, :math:`\\theta`]
+            1 represents a fixed condition, 0 represents a free conditon. 
+            e.x. [1,1,0]
+            A pin conneciton that's fixed in x/y but free in rotation.
+        label : str, optional
+            The label of the input node. 
+            labels are displayed in the plots. The default is ''.
         """
 
         fixity = self._convertFixityInput(fixity)
@@ -411,6 +464,10 @@ class Beam2D():
            New node 1: A vertical load of 10 is applied in beam units.
            [0., 0., 13]
            New node 2: A moment of 13 is applied in beam units.
+        label : str, optional
+            The label of the input node. 
+            labels are displayed in the plots. The default is ''.        
+        
         """
         
         loadID = len(self.pointLoads) + 1
@@ -430,26 +487,34 @@ class Beam2D():
         newLoad = PointLoad(pointLoad, x, nodeID)
         self.pointLoads.append(newLoad)                
          
-    def addVerticalLoad(self, x, Py, label=None):
+    def addVerticalLoad(self, x, Py, label=''):
         """
-        Adds a vertical load to the model at location x.
+        Adds a vertical load to the model at location x. If no node
+        exists at position x, a new node is added.
         Old loads at this point are deleted.
         
         Parameters
         ----------
         x : float
-            The x location to add a moment at.
+            The x location to add force at.
         Py : float
             The magnitude of the vertical load to be added at x.
+        label : str, optional
+            The label of the input node. 
+            labels are displayed in the plots. The default is ''.            
+            
         """   
 
         pointLoad = np.array([0., Py, 0.])
         self.addPointLoad(x, pointLoad, label)
         
-    def addMoment(self, x, M, label=None):
+    def addMoment(self, x, M, label=''):
         """
-        Adds a moment ot the model at location x.
+        Adds a moment ot the model at location x. If no node
+        exists at position x, a new node is added.
         Old loads at this point are deleted.
+        TODO:
+            State which direction positive is.
         
         Parameters
         ----------
@@ -457,14 +522,25 @@ class Beam2D():
             The x location to add a moment at.
         M : float
             The magnitude of the moment to be added at x.
+        label : str, optional
+            The label of the input node. 
+            labels are displayed in the plots. The default is ''.
         """        
         pointLoad = np.array([0.,0., M])
         self.addPointLoad(x, pointLoad, label)     
         
-    def addHorizontalLoad(self, x, Px, label=None):
+    def addHorizontalLoad(self, x, Px, label=''):
         """
-        Adds a horizontal point load at the model at location x.
+        Adds a horizontal point load at the model at location x. If no node
+        exists at position x, a new node is added.
         Old loads are deleted.
+        x : float
+            The x location to add force at.
+        Px : float
+            The magnitude of the vertical load to be added at x.        
+        label : str, optional
+            The label of the input node. 
+            labels are displayed in the plots. The default is ''.          
         """       
         
         pointLoads = np.array([Px, 0., 0.])
@@ -487,7 +563,6 @@ class Beam2D():
 
         """
         
-        
         for ii, node in enumerate(self.nodes):
             if xInput == node.x:
                 return ii
@@ -516,6 +591,7 @@ class Beam2D():
         
         defaultFixity = np.array([0,0,0], int)
         # genericPointLoad = np.array([0.,0.,0.], float)
+        distLoad = np.array(distLoad)
         
         if x1 not in self.nodeCoords:
             self.addNode(x1, defaultFixity)        
@@ -567,18 +643,19 @@ class Beam2D():
             A constantly distributed axial force.
         """
         distLoad = np.array([qx, 0.])
-        
         self.addDistLoad(x1, x2, distLoad)
 
     
-    def plot(self):
-        xcoords = np.array(list(self.nodeCoords))
-        y = np.zeros_like(xcoords)
-        plt.plot(xcoords, y)
-        plt.plot(xcoords, y, '.')
 
     """OuputPropreties"""
+    
+    
     def Fmax(self, index):
+        """
+        get the maximum and minimum internal force for teh beam along the 
+        appropriate axis. 0:x, 1:y, 2:z
+        
+        """
         Fmax = 0
         Fmin = 0
         for node in self.nodes:
@@ -596,10 +673,7 @@ class Beam2D():
     def getNodes(self):
         return self.nodes
     
-    
-    
-    
-    
+
     def getForces():
         pass
     
@@ -613,11 +687,73 @@ class Beam2D():
 
 
 
+def newEulerBeam2D(x2, x1 = 0, meshSize = 101):
+    """
+    Initializes a new 2D Euler beam. The beam will have no fixities or labels.
+
+    Parameters
+    ----------
+    x2 : float
+        The end position of the beam. If no x1 is provided, this is also the
+        length of the beam
+    x1 : float, optional
+        The start position of the beam. The default is 0.
+    meshSize : int, optional
+        The mesh size for the beam. This many nodes will be added between the 
+        points x1 and x2. The default is 101, which divides the beam into
+        100 even sections..
+
+    Returns
+    -------
+    EulerBeam2D : EulerBeam2D
+        The the beam intialized with the mesh of points between x1 and x2.
+
+    """
+    
+    if x2 <= x1:
+        raise Exception('x2 must be greater than x1')
+    
+    x = np.linspace(x1, x2, meshSize)  
+    
+    return EulerBeam2D(x)
+
+
+
 class EulerBeam2D(Beam2D):
-    # TODO: this is somewhat problematic as beam references the same list..
-    def __init__(self, xcoords = [], fixities = [], labels = [], 
-                 section = SectionBasic2D(), geomTransform = 'Linear'):
+    """
+    A creates a 2D Euler beam. Information about the beam is stored in a mesh
+    of nodes across the beam that are added by the user. Note that only output
+    information at the nodes will be contained in the analysis. 
+    
+    The units of the beam must form a consistent unit base for FEM
+    
+    Inherits from the base :py:class:`Beam2D` class.
+    
+    
+    Parameters
+    ----------
+    xcoords : list, optional
+        The x coodinates of nodes along the beam the beam. The default is [],
+        which starts with no nodes.
+    fixity : list
+        A list of fixities for the beam. Each fizity is a list of the input 
+        fixities for each possible degree of freedom. 
+        Each node will have three degree of freedoms; [x, y, :math:`\\theta`]
+        1 represents a fixed condition, 0 represents a free conditon. 
+        e.x. [1,1,0]
+        A pin conneciton that's fixed in x/y but free in rotation.
+    labels : list, optional
+        A list of labels for each node. The default is [], which gives no label
+        to each node.
+    section : Section2D, optional
+        The section to use in the anaysis. The default is SectionBasic2D().
         
+
+    """
+    # TODO: this is somewhat problematic as beam references the same list..
+    def __init__(self, xcoords = [], fixities = [], labels = [],
+                 section = SectionBasic2D()):
+
         # geomTransform has values 'Linear' or 'PDelta'
         self._initArrays()
         self.nodes = []
@@ -625,15 +761,12 @@ class EulerBeam2D(Beam2D):
                    
         self.Nnodes = 0
         NnewNodes = len(xcoords)
-       
-        if len(fixities) == 0:
-            fixities = [np.array([0., 0., 0.])] * NnewNodes
-        if len(fixities) != len(xcoords):
-            raise Exception('A fixity must be provided for each node.')
-            
+        
+        fixities = self._initFixities(fixities, NnewNodes)
+                
         if len(labels) == 0:
             labels = [None] * NnewNodes     
-            
+        
         if len(xcoords) != 0:
             self.addNodes(xcoords, fixities, labels)
         
@@ -642,9 +775,25 @@ class EulerBeam2D(Beam2D):
         self.materialPropreties = [section.A, section.E, section.Ixx]        
         self.plotter = None
         
-        self.geomTransform = geomTransform
         self.EleType = 'elasticBeamColumn'
+      
+    def _parseCoords(self, xcoords):
+        # If s
+        if type(xcoords) == float:
+            xcoords = [xcoords]
+        if len(xcoords) == 1:
+            xcoords = [0] + xcoords
+               
+      
+    def _initFixities(self, fixities, NnewNodes):
+        
            
+        if len(fixities) == 0:
+            fixities = [np.array([0., 0., 0.])] * NnewNodes
+        if len(fixities) != NnewNodes:
+            raise Exception('A fixity must be provided for each node.')
+        return fixities
+            
     
     def getMoment(self):
         """
@@ -737,37 +886,28 @@ class EulerBeam2D(Beam2D):
         return reactions
 
 
-
-# class Element():
-       
-#     def __init__(self, endNodes, eleLoad, ID):
-#         self.endNodes = endNodes
-#         self.eleLoad = eleLoad
-#         self.ID = ID
-
-
-
+@dataclass()
 class EleLoad:
-    def __init__(self, x1, x2, distLoad, label = None):
-        """
-        Representes a distrubted element load between two points px and py
+    """
+    Representes a distrubted element load between two points x1 x2. 
+    For 2D elements, distributed loads can either px or py.
 
-        Parameters
-        ----------
-        x1 : float
-            The start position.
-        x2 : float
-            The end position.
-        distLoad : TYPE
-            List of forces in [Px, Py].
-        label : str, optional
-            DESCRIPTION. The default is None.
+    Parameters
+    ----------
+    x1 : float
+        The start position.
+    x2 : float
+        The end position.
+    distLoad : list
+        For 2D, a List of forces in [Px, Py].
+    label : str, optional
+        A label for the elment load. The default is ''.
 
-        Returns
-        -------
-        None.
+    """    
+    
+    
+    def __init__(self, x1, x2, distLoad, label = ''):
 
-        """
         self.x1 = x1
         self.x2 = x2
         self.P = distLoad
@@ -775,39 +915,34 @@ class EleLoad:
 
 
 
-
+@dataclass()
 class PointLoad:
+    """
+    Representes a point load at locaiton x
+
+    Parameters
+    ----------
+    P : list[float]
+        List of forces. In 2D, as form [Px, Py, M].
+    x : float
+        The location of the point load.
+    nodeID : int
+        List of forces in [Px, Py].
+    label : str, optional
+        A label for the elment load. The default is ''.
+
+    """        
+    
     P = np.array([0.,0.,0.])
     nodeID = None
     
-    def __init__(self, P, x, nodeID = None, label = None):
+    def __init__(self, P, x, nodeID = None, label = ''):
         self.P = P
         self.x = x
         self.nodeID = nodeID
         self.label = label
         self.loadPattern = None
 
-    def setID(self, newID):
-        # print(newID)
+    def _setID(self, newID):
         self.nodeID = newID
         
-        
-
-class DistLoad():
-    
-    def __init__(self, ):
-    
-        pass
-       
-    
-class PlotBeam():
-
-    def __init__(self):
-        pass
-    
-    def initBeamPlot(self):
-        fig, ax = plt.subplots()
-    
-    
-
-
